<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/28/16
 * Time: 8:05 AM
 */

namespace qfq;

require_once(__DIR__ . '/../Constants.php');
require_once(__DIR__ . '/Sanitize.php');

class Support {

    /**
     * @param array $queryArray Empty or prefilled assoc array with url parameter
     * @param string $mode PARAM_T3_NO_ID, PARAM_T3_ALL
     */
    public static function appendTypo3ParameterToArray(array &$queryArray, $mode = PARAM_T3_ALL) {
//        if (isset($_GET['id']))
//            $queryArray['id'] = self::getCurrentPage();

        $store = Store::getInstance();

        if ($mode === PARAM_T3_ALL) {
            $queryArray['id'] = $store->getVar(TYPO3_PAGE_ID, STORE_TYPO3);
        }

        // TYPE
        $tmp = $store->getVar(TYPO3_PAGE_TYPE, STORE_TYPO3);
        if ($tmp !== false && $tmp != 0) {
            $queryArray['type'] = $tmp;
        }

        // Language
        $tmp = $store->getVar(TYPO3_PAGE_LANGUAGE, STORE_TYPO3);
        if ($tmp !== false && $tmp != 0) {
            $queryArray['L'] = $tmp;
        }
    }

    /**
     * Builds a urlencoded query string of an assoc array.
     *
     * @param array $queryArray
     * @return string Querystring (e.g.: id=23&type=99
     */
    public static function arrayToQueryString(array $queryArray) {
        $items = array();

        foreach ($queryArray as $key => $value) {
            if (is_int($key)) {
                $items[] = urlencode($value);
            } else {
                $items[] = $key . '=' . urlencode($value);
            }
        }

        return implode('&', $items);
    }

    /**
     * Extract Tag from $tag (might contain further attributes) and wrap it around $value. If $flagOmitEmpty==true && $value=='': return ''.
     *
     * @param $tag
     * @param $value
     * @param bool|false $omitIfValueEmpty
     * @return string
     */
    public static function wrapTag($tag, $value, $omitIfValueEmpty = false) {
        if ($omitIfValueEmpty && $value === "")
            return '';

        // a) <div class="container-fluid">, b) <label>
        $arr = explode(' ', $tag);

        $tagPlain = (count($arr) === 1) ? substr($arr[0], 1, strlen($arr[0]) - 2) : substr($arr[0], 1);
        $closing = '</' . $tagPlain . '>';

        return $tag . $value . $closing;
    }

    /**
     * Format's an attribute: $type=$value. If $flagOmitEmpty==true && $value=='': return ''.
     *
     * @param $type
     * @param $value
     * @param bool $flagOmitEmpty
     * @return string
     */
    public static function doAttribute($type, $value, $flagOmitEmpty = true) {
        if ($flagOmitEmpty && $value === "")
            return '';

        return $type . '="' . trim($value) . '" ';
    }

    /**
     * Wraps some $inner fragment with a CSS styled $tooltipText . CSS is configured in 'Resources/Public/qfq-jqw.css'.
     *
     * Based on: http://www.w3schools.com/howto/howto_css_tooltip.asp
     *
     * @param $before
     * @param $tooltipText
     * @return string
     */
    public static function appendTooltip($before, $tooltipText) {

        // addslashes, htmlentities & htmlspecialchars
        // str_replace('"', '&quot;', $tooltipText)

        return $before . "<img src='" . GFX_INFO . "' title=\"" . htmlentities($tooltipText) . "\">";
    }

    /**
     * Search for the parameter $needle in $haystack. The argumenst has to be seperated by ','.
     *
     * Returns the false if not found or index of found place. Be carefull: use unary operator to compare for 'false'
     *
     * @param $needle
     * @param $haystack
     * @return boolean     true if found, else false
     */
    public static function findInSet($needle, $haystack) {
        $arr = explode(',', $haystack);

        return array_search($needle, $arr) !== false;

    }

    /**
     * Converts a dateTime String to the english format:
     * 1.2.79 > 1979-02-01 00:00:00
     * 01.02.13 3:24 >  1979-02-01 03:24:00
     * 1.2.1979 14:21:5 > 1979-02-01 14:21:05
     *
     * @param $dateTimeString
     * @return string
     * @throws UserFormException
     */
    public static function dateTime2mysql($dateTimeString) {
        $dateRaw = '';
        $timeRaw = '';


//        const REGEXP_DATE_INT_ = '^\d{2,4}-\d{2}-\d{2}$';
//        const REGEXP_DATE_GER = '^\d{1,2}\.\d{1,2}\.\d{2}(\d{2})?$';
//        const REGEXP_TIME = '^\d{1,2}:\d{1,2}(:\d{1,2})?$';

        $tmpArr = explode(' ', $dateTimeString);
        switch (count($tmpArr)) {
            case 0:
                return '';

            case 1:
                if (strpos($tmpArr[0], ':') === false) {
                    $dateRaw = $tmpArr[0];
                } else {
                    $timeRaw = $tmpArr[0];
                }
                break;

            case 2:
                $dateRaw = $tmpArr[0];
                $timeRaw = $tmpArr[1];
                break;

            default:
                throw new UserFormException('Date/time format not recognised.', ERROR_DATE_TIME_FORMAT_NOT_RECOGNISED);
                break;
        }

        if ($dateRaw === '') {
            $date = '0000-00-00';
        } else {
            // International format: YYYY-MM-DD
            if (preg_match("/" . REGEXP_DATE_INT . "/", $dateRaw) === 1) {
                $date = $dateRaw;

                // German format: 1.1.01 - 11.12.1234
            } elseif (preg_match("/" . REGEXP_DATE_GER . "/", $dateRaw) === 1) {
                $tmpArr = explode('.', $dateRaw);

                if ($tmpArr[2] < 70) {
                    $tmpArr[2] = 2000 + $tmpArr[2];
                } elseif ($tmpArr[2] < 100) {
                    $tmpArr[2] = 1900 + $tmpArr[2];
                }
                $date = sprintf("%04d-%02d-%02d", $tmpArr[2], $tmpArr[1], $tmpArr[0]);

            } else {
                throw new UserFormException('Date/time format not recognised.', ERROR_DATE_TIME_FORMAT_NOT_RECOGNISED);
            }
        }

        if ($timeRaw === '') {
            $time = '00:00:00';
        } else {
            if (preg_match("/" . REGEXP_TIME . "/", $timeRaw) !== 1) {
                throw new UserFormException('Date/time format not recognised.', ERROR_DATE_TIME_FORMAT_NOT_RECOGNISED);
            }

            $tmpArr = explode(':', $timeRaw);
            switch (count($tmpArr)) {
                case 2:
                    $time = sprintf("%02d:%02d:00", $tmpArr[0], $tmpArr[1]);
                    break;
                case 3:
                    $time = sprintf("%02d:%02d:%02d", $tmpArr[0], $tmpArr[1], $tmpArr[2]);
                    break;
                default:
                    throw new UserFormException('Date/time format not recognised.', ERROR_DATE_TIME_FORMAT_NOT_RECOGNISED);
                    break;
            }
        }

        return $date . ' ' . $time;
    }

    /**
     * Encrypt curly braces by an uncommon string. Helps preventing unwished action on curly braces.
     *
     * @param $text
     * @return mixed
     */
    public static function encryptDoubleCurlyBraces($text) {
        $text = str_replace('{{', '#&@[[@_#', $text);
        $text = str_replace('}}', '#&@]]@_#', $text);

        return $text;
    }

    /**
     * Decrypt curly braces by an uncommon string. Helps preventing unwished action on curly braces
     *
     * @param $text
     * @return mixed
     */
    public static function decryptDoubleCurlyBraces($text) {
        $text = str_replace('#&@[[@_#', '{{', $text);
        $text = str_replace('#&@]]@_#', '}}', $text);

        return $text;
    }

    /**
     * @param int $length Length of the required hash string
     * @return string       A random alphanumeric hash
     */
    public static function randomAlphaNum($length) {
        $possible_characters = "abcdefghijklmnopqrstuvwxyz1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ";
        $string = "";
        while (strlen($string) < $length) {
            $string .= substr($possible_characters, rand() % (strlen($possible_characters)), 1);
        }

        return ($string);
    }

    /**
     * @param $url
     * @param $param
     * @return string
     */
    public static function concatUrlParam($url, $param) {
        if ($param == '')
            return $url;

        $token = (strpos($url, '?') === false) ? '?' : '&';
        return $url . $token . $param;
    }

    /**
     * @param $arr
     * @param $index
     * @param string $value
     */
    public static function setIfNotSet(&$arr, $index, $value = '') {

        if (!isset($arr[$index]))
            $arr[$index] = $value;
    }
}