<?PHP
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2010 Glowbase GmbH <support@glowbase.com>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace qfq;

//use qfq;

require_once(__DIR__ . '/Define.php');
require_once(__DIR__ . '/Utils.php');
require_once(__DIR__ . '/../store/Store.php');
require_once(__DIR__ . '/../store/Sip.php');
require_once(__DIR__ . '/../exceptions/UserReportExtension.php');

/*
 * u:url
 * m:mailto
 * p:page
 * 
 * t:text
 * a:AltText
 * o:ToolTip
 * 
 * r:render   
 * g:target
 * c:class  [n|i|e|<class>]
 * q:question  <text>
 * e:encryption 0|1
 * 
 * E:edit
 * N:new
 * D:delete
 * H:Help
 * I:information
 * M:Magnifier
 * B:bullet
 * P:picture       [file]
 * C:checkbox    [name]
 * R:right 
 * h:hash  
 *
 * A:    A:[u:p:m]
 * G:    G:[N|..]
 */


const NAME_URL = 'url';
const NAME_MAIL = 'mail';
const NAME_PAGE = 'page';
const NAME_TEXT = 'text';
const NAME_ALT_TEXT = 'altText';
const NAME_TOOL_TIP = 'toolTip';
const NAME_TOOL_TIP_JS = 'toolTipJs';
const NAME_IMAGE = 'image';
const NAME_IMAGE_TITLE = 'imageTitle';
const NAME_GLYPH = 'glyph';
const NAME_GLYPH_TITLE = 'glyphTitle';
const NAME_DELETE = 'delete';
const NAME_RENDER = 'render';
const NAME_TARGET = 'target';
const NAME_LINK_CLASS = 'linkClass';
const NAME_LINK_CLASS_DEFAULT = 'linkClassDefault';
const NAME_QUESTION = 'question';
const NAME_ENCRYPTION = 'encryption';
const NAME_HASH = 'hash';
const NAME_URL_PARAM = 'param';
const NAME_RIGHT_PICTURE_POSITION = 'picturePosition';

const NO_CLASS = 'no_class';

/**
 * Class Link
 * @package qfq
 */
class Link {

    //TODO: delete muss noch implementiert werden
    private $delete = '';

    /**
     * @var Sip
     */
    private $sip = null;

    /**
     * @var Store
     */
    private $store = null;

    // Simulate global variable: will be set much earlier in other functions. Will be shown in error messages.
    private $fr_error;

    /**
     * @var Utils
     */
    private $utils;
    /**
     * @var array
     */
    private $renderControl = array();
    private $linkClassSelector = array("i" => "internal", "e" => "external");
    private $cssLinkClassInternal = '';
    private $cssLinkClassExternal = '';

    private $callTable = [
        'u' => 'buildUrl',
        'm' => 'buildMail',
        'p' => 'buildPage',
        'o' => 'buildToolTip',
        'P' => 'buildPicture',
        'B' => 'buildBullet',
        'C' => 'buildCheck',
        'D' => 'buildDelete',
        'E' => 'buildEdit',
        'H' => 'buildHelp',
        'I' => 'buildInfo',
        'N' => 'buildNew',
        'S' => 'buildShow',
    ];

    private $tableVarName = [
        'u' => NAME_URL,
        'm' => NAME_MAIL,
        'p' => NAME_PAGE,
        't' => NAME_TEXT,
        'a' => NAME_ALT_TEXT,
        'o' => NAME_TOOL_TIP,
        'P' => NAME_IMAGE,
        'B' => NAME_IMAGE,
        'C' => NAME_IMAGE,
        'D' => NAME_IMAGE,
        'E' => NAME_IMAGE,
        'H' => NAME_IMAGE,
        'I' => NAME_IMAGE,
        'N' => NAME_IMAGE,
        'S' => NAME_IMAGE,
        'r' => NAME_RENDER,
        'g' => NAME_TARGET,
        'c' => NAME_LINK_CLASS,
        'q' => NAME_QUESTION,
        'e' => NAME_ENCRYPTION,
        'h' => NAME_HASH,
        'U' => NAME_URL_PARAM,
        'R' => NAME_RIGHT_PICTURE_POSITION,
    ];

    private $varsDefault = array();

    /**
     * __construct
     *
     * @param $fr_error
     * @param Sip $sip
     * @param bool $phpUnit
     */
    public function __construct($fr_error, Sip $sip, $phpUnit = false) {
        $this->fr_error = $fr_error;
        $this->sip = $sip;
        $this->store = Store::getInstance('', $phpUnit);
        $this->cssLinkClassInternal = $this->store->getVar(SYSTEM_CSS_LINK_CLASS_INTERNAL, STORE_SYSTEM);
        $this->cssLinkClassExternal = $this->store->getVar(SYSTEM_CSS_LINK_CLASS_EXTERNAL, STORE_SYSTEM);
        $this->utils = new Utils();

        $this->varsDefault[NAME_QUESTION] = 'Please confirm';
//        $this->varsDefault[NAME_PAGE] = $this->store->getVar(TYPO3_PAGE_ID, STORE_TYPO3);
        $this->varsDefault[NAME_ENCRYPTION] = '0';
        $this->varsDefault[NAME_RIGHT_PICTURE_POSITION] = 'l';
        $this->varsDefault[NAME_RENDER] = 0;
        $this->varsDefault[NAME_HASH] = 0;

        /*
         * mode:
         * 0: no output
         * 1: text
         * 2: url
         * 3: <a href=url>url</a>
         * 4: <a href=url>Text</a>
         *
         *  r=render mode, u=url, t:text and/or image.
         *
         *                  [r][u][t] = mode
         */

        $this->renderControl[0][0][0] = 0;
        $this->renderControl[0][0][1] = 0;
        $this->renderControl[0][1][0] = 3;
        $this->renderControl[0][1][1] = 4;

        $this->renderControl[1][0][0] = 0;
        $this->renderControl[1][0][1] = 1;
        $this->renderControl[1][1][0] = 3;
        $this->renderControl[1][1][1] = 4;

        $this->renderControl[2][0][0] = 0;
        $this->renderControl[2][0][1] = 0;
        $this->renderControl[2][1][0] = 0;
        $this->renderControl[2][1][1] = 4;

        $this->renderControl[3][0][0] = 0;
        $this->renderControl[3][0][1] = 1;
        $this->renderControl[3][1][0] = 2;
        $this->renderControl[3][1][1] = 1;

        $this->renderControl[4][0][0] = 0;
        $this->renderControl[4][0][1] = 1;
        $this->renderControl[4][1][0] = 2;
        $this->renderControl[4][1][1] = 2;

        $this->renderControl[5][0][0] = 0;
        $this->renderControl[5][0][1] = 0;
        $this->renderControl[5][1][0] = 0;
        $this->renderControl[5][1][1] = 0;

    }

    /**
     * Build the whole link
     *
     * @param string $str Qualifier with params. 'report'-syntax. F.e.:  A:u:www.example.com|G:P:home.gif|t:Home"
     * @return string     The complete HTML encoded Link like <a href='http://example.com' class='external'><img src='iconf.gif' title='help text'>Description</a>
     * @throws UserReportException
     */
    public function renderLink($str) {
        $link = '';

        $vars = $this->initVars();

        // str="u:http://www.example.com|c:i|t:delete"
        $param = explode("|", $str);

        // Parse all parameter, fill variables
        foreach ($param as $item) {

            if ($item == '') {
                continue;
            }

            // set class defaults
            $this->parseItem($vars, $item);
        }

        $necessaryDefaults = [NAME_RENDER, NAME_RIGHT_PICTURE_POSITION, NAME_HASH, NAME_ENCRYPTION];
        foreach ($necessaryDefaults AS $keyName) {
            if ($vars[$keyName] == '') {
                $vars[$keyName] = $this->varsDefault[$keyName];
            }
        }


        // if there is no url or mailto definition: {{global.pageId}}
//        if ($vars[NAME_URL] == '' && $vars[NAME_MAIL] == '') {
//            $vars[NAME_URL] = "?" . $this->store->getVar(TYPO3_PAGE_ID, STORE_TYPO3);
//        }

        $this->doCssClass($vars);

        // Set default tooltip
        if ($vars[NAME_TOOL_TIP] == '' && $vars[NAME_GLYPH_TITLE] !== '') {
            $vars[NAME_TOOL_TIP] = $vars[NAME_GLYPH_TITLE];
        }

        $htmlUrl = $this->doAnchor($vars);
        $htmlImage = $this->doHtmlImageGlyph($vars);

// Compose Image & Text
        if ($htmlImage != '') {
            $distance = ' ';
            $vars[NAME_TEXT] = ($vars[NAME_RIGHT_PICTURE_POSITION] == "l") ? $htmlImage . $distance . $vars[NAME_TEXT] : $vars[NAME_TEXT] . $distance . $htmlImage;
        }

// ToolTip
        $extraSpan = ['', ''];
        if ($vars[NAME_TOOL_TIP] !== '') {
            $extraSpan[0] = "<span " . $vars[NAME_TOOL_TIP_JS][0] . ">" . $vars[NAME_TOOL_TIP_JS][1];
            $extraSpan[1] = "</span>";
        }

// Create 'fake' modes for encrypted 'mailto'
        $prefix = "";
        if ($vars[NAME_MAIL] != '') {
            $prefix = "1";
            $vars[NAME_URL] = "dummy";
        }
// Create 'fake' mode for ajax delete
        if ($vars[NAME_DELETE]) {
            $prefix = "2";
        }
// Compose URL
// get Render Mode via Array renderControl
        $modeRender = $vars[NAME_RENDER];
        $modeUrl = $vars[NAME_URL] ? 1 : 0;
        $modeText = $vars[NAME_TEXT] ? 1 : 0;

        if (isset($this->renderControl[$modeRender][$modeUrl][$modeText])) {
            $mode = $prefix . $this->renderControl[$modeRender][$modeUrl][$modeText];
        } else {
            throw new UserReportException ("Undefined combination of 'rendering mode' / 'url' / 'text': " .
                $modeRender . ' / ' . $modeUrl . ' / ' . $modeText, ERROR_UNDEFINED_RENDER_CONTROL_COMBINATION);
        }

// 0-4 URL, plain email
// 10-14 encrypted email
        switch ($mode) {
            // 0: No Output
            case '0':
            case '10':
            case '20':
            $link = "";
                break;

            // 1: 'text'
            case '1':
                $link = $extraSpan[0] . $vars[NAME_TEXT] . $extraSpan[1];
                break;
            case '11':
                $link = $extraSpan[0] . $this->encryptMailtoJS($vars, false) . $extraSpan[1];
                break;

            // 2: 'url'
            case '2':
                $link = $extraSpan[0] . $vars[NAME_URL] . $extraSpan[1];
                break;
            case '12':
                $link = $extraSpan[0] . $this->encryptMailtoJS($vars, false) . $extraSpan[1];
                break;

            // 3: <a href=url>url</a>
            case '3':
                $link = $htmlUrl . $vars[NAME_URL] . '</a>' . $vars[NAME_TOOL_TIP_JS][1];
                break;
            case '13':
                $vars[NAME_TEXT] = $vars[NAME_MAIL];
                $link = $this->encryptMailtoJS($vars, TRUE);
                break;

            // 4: <a href=url>Text</a>
            case '4':
                $link = $htmlUrl . $vars[NAME_TEXT] . '</a>' . $vars[NAME_TOOL_TIP_JS][1];
                break;
            case '14':
                $link = $this->encryptMailtoJS($vars, TRUE);
                break;
            case '21':
            case '22':
            case '23':
            case '24':
            //TODO: Alter Code, umstellen auf JS Client von RO. Vorlage koennte 'Delete' in Subrecord sein.
            $link = "<a href=\"javascript: void(0);\" onClick=\"var del = new FR.Delete({recordId:'',hash:'',forward:'" .
                $vars[NAME_PAGE] . "'});\" " . $vars[NAME_LINK_CLASS] . ">" . $vars[NAME_TEXT] . "</a>";
        }

        return $link;

    }

    /**
     * Cleans the standard vars used every time to render a link.
     *
     * @return array
     */
    private function initVars() {

        $vars = array();

        $vars[NAME_MAIL] = '';
        $vars[NAME_URL] = '';
        $vars[NAME_PAGE] = '';

        $vars[NAME_TEXT] = '';
        $vars[NAME_ALT_TEXT] = '';
        $vars[NAME_IMAGE] = '';
        $vars[NAME_IMAGE_TITLE] = '';
        $vars[NAME_GLYPH] = '';
        $vars[NAME_GLYPH_TITLE] = '';
        $vars[NAME_QUESTION] = '';
        $vars[NAME_TARGET] = '';
        $vars[NAME_TOOL_TIP] = '';
        $vars[NAME_TOOL_TIP_JS] = ['', ''];
        $vars[NAME_URL_PARAM] = '';

        $vars[NAME_RENDER] = '';
        $vars[NAME_RIGHT_PICTURE_POSITION] = '';
        $vars[NAME_HASH] = '';
        $vars[NAME_ENCRYPTION] = '';
        $vars[NAME_DELETE] = '';

        $vars[NAME_LINK_CLASS] = ''; // class name
        $vars[NAME_LINK_CLASS_DEFAULT] = ''; // Depending of 'as page' or 'as url'. Only used if class is not explizit set.

        return $vars;
    }

    /**
     * Parse Item of link string, fill class global variables.
     *
     * @param array $vars
     * @param $item
     * @throws CodeException
     * @throws UserReportException
     */
    private function parseItem(array &$vars, $item) {

        $arr = explode(":", $item, 2);
        $key = isset($arr[0]) ? $arr[0] : '';
        $value = isset($arr[1]) ? $arr[1] : '';

        // Superclass
        if ($key === 'A' || $key === 'G') {
            $this->parseItem($vars, $value);
            return;
        }

        if (!isset($this->tableVarName[$key])) {
            throw new UserReportException ("Unknown link qualifier: '$key'", ERROR_UNKNOWN_LINK_QUALIFIER);
        }

        $keyName = $this->tableVarName[$key];

        // A few keys do not have necessarily a value: fake the definition by manual creating a value.
        if ($value == '') {
            switch ($key) {
                case 'R':
                    $value = 'r';
                    break;
                case 'h':
                    $value = '1';
                    break;
                case 'p':
                    $value = $this->store->getVar(TYPO3_PAGE_ID, STORE_TYPO3); // If no pageid|pagealias is defined, take current page
                    break;
                default:
                    break;
            }
        }

        // Defaults
        if ($value === '' && isset($this->varsDefault[$keyName])) {
            $value = $this->varsDefault[$keyName];
        }

        //  Check for empty values. Respect: some keys are allowed to be empty.
        if ($value === '' && strpos('uENDHIS', $key) === false) {
            throw new UserReportException ("Missing value for '$key'", ERROR_MISSING_VALUE);
        }

        // Value already defined?
        if ($vars[$keyName] !== '') {
            throw new UserReportException ("Multiple definitions for key '$key'", ERROR_MULTIPLE_DEFINITION);
        }

        // Store value
        $vars[$keyName] = $value;

        if ($vars[NAME_URL] && $vars[NAME_MAIL]) {
            throw new UserReportException ("Multiple URL / PAGE and MAILTO definition", ERROR_MULTIPLE_URL_PAGE_MAILTO_DEFINITION);
        }

        // Call special function
        if (isset($this->callTable[$key])) {
            $call = $this->callTable[$key];
            $this->$call($vars, $keyName, $value);
        }
    }

    /**
     * Parse CSS Class Settings
     *
     * @param array $vars
     * @return    void
     */
    private function doCssClass(array &$vars) {
        $class = '';

        switch ($vars[NAME_LINK_CLASS]) {
            case 'n':
                $vars[NAME_LINK_CLASS] = '';
                break;
            case 'i':
            case 'e':
            $vars[NAME_LINK_CLASS] = $this->linkClassSelector[$vars[NAME_LINK_CLASS]];
                break;
            default:
                $vars[NAME_LINK_CLASS] = ($vars[NAME_LINK_CLASS] == '') ? $vars[NAME_LINK_CLASS_DEFAULT] : $vars[NAME_LINK_CLASS];
                break;
        }

        if ($vars[NAME_LINK_CLASS] === NO_CLASS) {
            $vars[NAME_LINK_CLASS] = '';
        }
    }

    /**
     * Create the HTML anchor.
     * - <a href="mailto:info@example.com" title=".." class="..">
     * - <a href="http://example.com" title=".." class="..">
     * - If $this->modeHash is set, create a hash (sip)
     *
     * @param array $vars
     * @return string
     * @throws UserReportException
     */
    private function doAnchor(array &$vars) {
        $attributes = '';
        // build URL
        $anchorTitle = '';

        // Link: URL
        if ($vars[NAME_URL] !== '') {

            if ($vars[NAME_HASH] === "1") {

                $paramArray = $this->sip->queryStringToSip(Support::concatUrlParam($vars[NAME_URL], $vars[NAME_URL_PARAM]), RETURN_ARRAY);
                $vars[NAME_URL] = $paramArray['_url'];
                if ($this->store->getVar(SYSTEM_SHOW_DEBUG_INFO, STORE_SYSTEM) === 'yes') {

                    $vars[NAME_TOOL_TIP] .= PHP_EOL . PHP_EOL . OnArray::toString($paramArray, ' = ', PHP_EOL, "'");
                    $this->buildToolTip($vars, 'o', $vars[NAME_TOOL_TIP]);
                }

            } else {

                if ($vars[NAME_URL_PARAM] != '') {
                    $vars[NAME_URL] = Support::concatUrlParam($vars[NAME_URL], $vars[NAME_URL_PARAM]);
                }
            }
        }

        // Link: MAILTO
        if ($vars[NAME_MAIL] !== '') {

            // If there is no encryption: handle the mailto as an ordinary URL
            if ($vars[NAME_ENCRYPTION] === '0') {
                $vars[NAME_URL] = "mailto:" . $vars[NAME_MAIL];
                $vars[NAME_MAIL] = "";
            } else {
                throw new UserReportException ("Oops, sorry: encrypted mail not implemented ...", ERROR_NOT_IMPLEMENTED);
            }
        }

        if ($vars[NAME_GLYPH] !== '') {
            $vars[NAME_LINK_CLASS] .= ' btn btn-default ';

//            if ($vars[NAME_GLYPH_TITLE] !== '') {
//                $anchorTitle = $vars[NAME_GLYPH_TITLE];
//            }
        }

        $attributes .= Support::doAttribute('href', $vars[NAME_URL]);
        $attributes .= Support::doAttribute('class', $vars[NAME_LINK_CLASS]);
        $attributes .= Support::doAttribute('target', $vars[NAME_TARGET]);
//        $attributes .= Support::doAttribute('title', $anchorTitle);
        $attributes .= Support::doAttribute('title', $vars[NAME_TOOL_TIP]);

        if ($vars[NAME_QUESTION]) {
            $attributes .= Support::doAttribute('onclick', 'confirm(\'' . $vars[NAME_QUESTION] . '\')');
        }

//        $anchor = '<a ' . $attributes . $vars[NAME_TOOL_TIP_JS][0] . '>';
        $anchor = '<a ' . $attributes . '>';

        return ($anchor);
    }

    //
//<a class="btn btn-default" href="index.php?id=2&s=56fbdd30a0cd2" title='comment'>
//   <span class='glyphicon glyphicon-plus'></span>
//</a>
//
//<a href="index.php?id=2&s=56fb7f7703692" class="internal" >
//  <img src="typo3conf/ext/qfq/Resources/Public/icons/edit.gif" title="Edit" >
//</a>

    /**
     * Create a ToolTip: $toolTip[0] and $toolTip[1] have to inserted in HTML code accordingly.
     * $vars[NAME_TOOL_TIP_JS][0]: JS to show '$toolTip[1]'.
     * $vars[NAME_TOOL_TIP_JS][1]: '<span>...</span>' with the tooltip text.
     *
     * @param $vars
     * @param $key
     * @param $value
     */
    private function buildToolTip(&$vars, $key, $value) {
        static $count = 0;

        $toolTipIndex = 'tooltip.' . $GLOBALS["TSFE"]->currentRecord . '.' . ++$count;
        $vars[NAME_TOOL_TIP_JS] = array();

        // Expample: <img src="fileadmin/icons/bullet-gray.gif" onmouseover="document.getElementById('gm167979').style.
        // display='block';" onmouseout="document.getElementById('gm167979').style.display='none';" />
        $vars[NAME_TOOL_TIP_JS][0] = " onmouseover=\"document.getElementById('" . $toolTipIndex .
            "').style.display='block';\" onmouseout=\"document.getElementById('" . $toolTipIndex . "').style.display='none';\"";

        // Example: <span id="gm167979" style="display:none; position:absolute; border:solid 1px black; background-color:#F9F3D0;
        // padding:3px;">My pesonal tooltip</span>
        $vars[NAME_TOOL_TIP_JS][1] = '<span id="' . $toolTipIndex .
            '" style="display:none; position:absolute; border:solid 1px black; background-color:#F9F3D0; padding:3px;">' .
            $value . '</span>';

        return;
    }

    /**
     * Create Image HTML Tag
     *
     * @param array $vars
     * @return string
     */
    private function doHtmlImageGlyph(array $vars) {
        $tags = '';
        $html = '';

        // Build Image
        if ($vars[NAME_IMAGE] !== '') {
            $tags .= Support::doAttribute('alt', $vars[NAME_ALT_TEXT]);
            $tags .= Support::doAttribute('src', $vars[NAME_IMAGE]);
            $tags .= Support::doAttribute('title', $vars[NAME_IMAGE_TITLE]);
            $html .= '<img ' . $tags . '>';
        }

        if ($vars[NAME_GLYPH] !== '') {
            $tags = Support::doAttribute('class', 'glyphicon ' . $vars[NAME_GLYPH]);
            $html .= Support::wrapTag('<span ' . $tags . '>', '', false);
        }

        return $html;
    }

    /**
     *  Encrypt the mailto address via JS.
     *  Email address protected against email crawler (as long as they don't interpret JS).
     *
     *    <script language=javascript><!--
     *    var mm1 = name
     *    var mm2 = @domain.
     *    var ... = tld
     *    document.write("<a href=" + "mail" + "to:" + mm1 + mm2 + ... + ">" + name + "</a>")
     *    document.write("<a href=" + "mail" + "to:" + mm1 + mm2 + ... + ">" + @domain. + "</a>")
     *    document.write("<a href=" + "mail" + "to:" + mm1 + mm2 + ... + ">" + tld + "</a>")
     *    //--></script>';
     *
     * @param array $vars
     * @param bool|TRUE $href TRUE: create a '<a>',   false: just encrypt or show the email, no link.
     * @return string
     */
    private function encryptMailtoJS(array $vars, $href = TRUE) {

        // Split $mailto
        $tmp = $this->splitAndAddDelimter($vars[NAME_MAIL], "@");
        $arr = array_merge($this->splitAndAddDelimter($tmp[0], "."), $this->splitAndAddDelimter($tmp[1], "."));

        $tt = "<script language=javascript><!--" . chr(10);
        $ii = 0;
        if ($href) {
            $dw = 'document.write("<a href=" + "mail" + "to:"';
            foreach ($arr as $value) {
                // Setup JS Variables
                $tt .= 'var mm' . $ii . '= "' . $value . '"' . chr(10);
                // Compose $dw (documentwrite statement)
                $dw .= ' + mm' . $ii++;
            }
            $dw .= ' + "' . $vars[NAME_LINK_CLASS] . str_replace('"', '\\"', $vars[NAME_TOOL_TIP_JS][0]) . '>"';
            $closeDw = '"</a>")';
        } else {
            $dw = 'document.write(';
            $closeDw = ')';
        }

        // Wrap mailto around text
        if ($vars[NAME_MAIL] == $vars[NAME_TEXT]) {
            // Text is an email: wrap every single part of mailto.
            $ii = 0;
            foreach ($arr as $value) {
                $tt .= $dw . " + mm" . $ii++ . ' + ' . $closeDw . chr(10);
            }
        } else {
            // Single wrap text
            $tt .= $dw . ' + "' . $vars[NAME_TEXT] . '" + ' . $closeDw . chr(10);
        }

        $tt .= '//--></script>';
        if ($href) $tt .= $vars[NAME_TOOL_TIP_JS][1];

        return ($tt);
    }

    /**
     * Split a string around the $delimiter.
     *
     * Append the delimiter to each part except the last one.
     *
     * @param $mailto
     * @param $delimiter
     * @return array
     */
    private function splitAndAddDelimter($mailto, $delimiter) {
        //TODO: Ich verstehe die Funktion nicht - funktioniert das hier wirklich?
        $value = '';

        $arr = explode($delimiter, $mailto);            // split string

        foreach ($arr as $key => $value) {
            $arr[$key] = $value . $delimiter;
        }

        if (isset($key))
            $arr[$key] = $value;

        return ($arr);
    }

    /**
     * Called by $this->callTable
     *
     * @param $vars
     * @param $key
     * @param $value
     */
    private function buildUrl(&$vars, $key, $value) {
        $vars[NAME_LINK_CLASS_DEFAULT] = $this->cssLinkClassExternal;
    }

    /**
     * Called by $this->callTable
     *
     * @param $vars
     * @param $key
     * @param $value
     */
    private function buildMail(&$vars, $key, $value) {
        $vars[NAME_LINK_CLASS_DEFAULT] = $this->cssLinkClassExternal;
        $vars[NAME_LINK_CLASS_DEFAULT] = NO_CLASS;
    }

    /**
     * Called by $this->callTable
     *
     * @param $vars
     * @param $key
     * @param $value
     * @throws UserReportException
     */
    private function buildPage(&$vars, $key, $value) {

        if ($vars[NAME_URL] != '') {
            throw new UserReportException ("Multiple definitions for key 'p'", ERROR_MULTIPLE_DEFINITION);
        }

        if (substr($value, 0, 3) !== 'id=') {
            $value = 'id=' . $value;
        }

        $vars[NAME_URL] = "?" . $value;
        $vars[NAME_LINK_CLASS_DEFAULT] = $this->cssLinkClassInternal;
    }

    /**
     * Called by $this->callTable
     *
     * @param $vars
     * @param $key
     * @param $value
     */
    private function buildPicture(&$vars, $key, $value) {
        $vars[NAME_ALT_TEXT] = "Grafic: " . $value;
        $vars[NAME_IMAGE_TITLE] = $value;
        $vars[NAME_LINK_CLASS_DEFAULT] = NO_CLASS;
    }

    /**
     * Called by $this->callTable
     *
     * @param $vars
     * @param $key
     * @param $value
     */
    private function buildBullet(&$vars, $key, $value) {
        $vars[NAME_IMAGE] = PATH_ICONS . "bullet-" . $value . '.gif';
        $vars[NAME_IMAGE_TITLE] = $value;
        $vars[NAME_LINK_CLASS_DEFAULT] = NO_CLASS;
    }

    /**
     * Called by $this->callTable
     *
     * @param $vars
     * @param $key
     * @param $value
     */
    private function buildCheck(&$vars, $key, $value) {
        $vars[NAME_IMAGE] = PATH_ICONS . "checked-" . $value . '.gif';
        $vars[NAME_IMAGE_TITLE] = $value;
        $vars[NAME_LINK_CLASS_DEFAULT] = NO_CLASS;
    }

    /**
     * Called by $this->callTable
     *
     * @param $vars
     * @param $key
     * @param $value
     */
    private function buildDelete(&$vars, $key, $value) {
//        $vars[NAME_IMAGE] = PATH_ICONS . 'delete.gif';
//        $vars[NAME_IMAGE_TITLE] = "Delete";
        $vars[NAME_DELETE] = true;

        $vars[NAME_GLYPH] = GLYPH_ICON_DELETE;
        $vars[NAME_GLYPH_TITLE] = "Delete";
        $vars[NAME_LINK_CLASS_DEFAULT] = NO_CLASS;

        // Include Extjs library
        $this->utils->loadJSlib($this->fr_error);
    }

    /**
     * Called by $this->callTable
     *
     * @param $vars
     * @param $key
     * @param $value
     */
    private function buildEdit(&$vars, $key, $value) {
//        $vars[NAME_IMAGE] = PATH_ICONS . 'edit.gif';
//        $vars[NAME_IMAGE_TITLE] = "Edit";
        $vars[NAME_GLYPH] = GLYPH_ICON_EDIT;
        $vars[NAME_GLYPH_TITLE] = "Edit";
        $vars[NAME_LINK_CLASS_DEFAULT] = NO_CLASS;
    }

    /**
     * Called by $this->callTable
     *
     * @param $vars
     * @param $key
     * @param $value
     */
    private function buildHelp(&$vars, $key, $value) {
//        $vars[NAME_IMAGE] = PATH_ICONS . 'help.gif';
//        $vars[NAME_IMAGE_TITLE] = "Help";
        $vars[NAME_GLYPH] = GLYPH_ICON_HELP;
        $vars[NAME_GLYPH_TITLE] = "Help";
        $vars[NAME_LINK_CLASS_DEFAULT] = NO_CLASS;

    }

    /**
     * Called by $this->callTable
     *
     * @param $vars
     * @param $key
     * @param $value
     */
    private function buildInfo(&$vars, $key, $value) {
//        $vars[NAME_IMAGE] = PATH_ICONS . 'info.gif';
//        $vars[NAME_IMAGE_TITLE] = "Information";

        $vars[NAME_GLYPH] = GLYPH_ICON_INFO;
        $vars[NAME_GLYPH_TITLE] = "Information";
        $vars[NAME_LINK_CLASS_DEFAULT] = NO_CLASS;

    }

    /**
     * Called by $this->callTable
     *
     * @param $vars
     * @param $key
     * @param $value
     */
    private function buildNew(&$vars, $key, $value) {
//        $vars[NAME_IMAGE] = PATH_ICONS . 'new.gif';
//        $vars[NAME_IMAGE_TITLE] = "New";

        $vars[NAME_GLYPH] = GLYPH_ICON_NEW;
        $vars[NAME_GLYPH_TITLE] = "New";
        $vars[NAME_LINK_CLASS_DEFAULT] = NO_CLASS;

    }

    /**
     * Called by $this->callTable
     *
     * @param $vars
     * @param $key
     * @param $value
     */
    private function buildShow(&$vars, $key, $value) {
//        $vars[NAME_IMAGE] = PATH_ICONS . 'show.gif';
//        $vars[NAME_IMAGE_TITLE] = "Details";

        $vars[NAME_GLYPH] = GLYPH_ICON_SHOW;
        $vars[NAME_GLYPH_TITLE] = "Details";
        $vars[NAME_LINK_CLASS_DEFAULT] = NO_CLASS;

    }

}