<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 3/31/16
 * Time: 10:06 PM
 */

namespace qfq;

require_once(__DIR__ . '/../../qfq/report/Link.php');
require_once(__DIR__ . '/../../qfq/store/Store.php');
require_once(__DIR__ . '/../../qfq/store/Sip.php');


class LinkTest extends \PHPUnit_Framework_TestCase {

    /**
     * @var Sip
     */
    private $sip = null;

    /**
     * @var Store
     */
    private $store = null;

    /**
     * @throws SyntaxReportException
     */
    public function testLinkUrlBasic() {
        $link = new Link(null, $this->sip, true);

        $result = $link->renderLink('');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com');
        $this->assertEquals('<a href="http://example.com" class="external" >http://example.com</a>', $result);

        $result = $link->renderLink('u:http://example.com|t:Hello world');
        $this->assertEquals('<a href="http://example.com" class="external" >Hello world</a>', $result);
    }

    /**
     * @expectedException \qfq\UserReportException
     *
     */
    public function testLinkUrlBasicExceptionDouble() {
        $link = new Link(null, $this->sip, true);

        $result = $link->renderLink('u:http://example.com|u:http://new.org');
    }


    /**
     * @throws SyntaxReportException
     */
    public function testLinkPageBasic() {
        $link = new Link(null, $this->sip, true);
        Store::setVar(TYPO3_PAGE_ID, 'firstPage', STORE_TYPO3);

        $result = $link->renderLink('p');
        $this->assertEquals('<a href="?id=firstPage" class="internal" >?id=firstPage</a>', $result);

        $result = $link->renderLink('p:');
        $this->assertEquals('<a href="?id=firstPage" class="internal" >?id=firstPage</a>', $result);

        $result = $link->renderLink('p:secondPage');
        $this->assertEquals('<a href="?id=secondPage" class="internal" >?id=secondPage</a>', $result);

        $result = $link->renderLink('p:id=secondPage');
        $this->assertEquals('<a href="?id=secondPage" class="internal" >?id=secondPage</a>', $result);

        $result = $link->renderLink('p:secondPage|t:Hello world');
        $this->assertEquals('<a href="?id=secondPage" class="internal" >Hello world</a>', $result);
    }

    /**
     * @throws SyntaxReportException
     */
    public function testLinkMailBasic() {
        $link = new Link(null, $this->sip, true);

        $result = $link->renderLink('m:john@doe.com');
        $this->assertEquals('<a href="mailto:john@doe.com" >mailto:john@doe.com</a>', $result);

        $result = $link->renderLink('m:john@doe.com|t:John Doe');
        $this->assertEquals('<a href="mailto:john@doe.com" >John Doe</a>', $result);
    }

    /**
     * @expectedException \qfq\UserReportException
     *
     */
    public function testLinkMailBasicExceptionMissing1() {
        $link = new Link(null, $this->sip, true);

        $result = $link->renderLink('m');
    }

    /**
     * @expectedException \qfq\UserReportException
     *
     */
    public function testLinkMailBasicExceptionMissing2() {
        $link = new Link(null, $this->sip, true);

        $result = $link->renderLink('m:');
    }

    public function testRenderModeUrl() {
        $link = new Link(null, $this->sip, true);

        // r: default (0)
        $result = $link->renderLink('u');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com');
        $this->assertEquals('<a href="http://example.com" class="external" >http://example.com</a>', $result);

        $result = $link->renderLink('t:Example');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example');
        $this->assertEquals('<a href="http://example.com" class="external" >Example</a>', $result);

        // r: 0
        $result = $link->renderLink('u|r:0');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|r:0');
        $this->assertEquals('<a href="http://example.com" class="external" >http://example.com</a>', $result);

        $result = $link->renderLink('t:Example|r:0');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:0');
        $this->assertEquals('<a href="http://example.com" class="external" >Example</a>', $result);

        // r: 1
        $result = $link->renderLink('u|r:1');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|r:1');
        $this->assertEquals('<a href="http://example.com" class="external" >http://example.com</a>', $result);

        $result = $link->renderLink('t:Example|r:1');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:1');
        $this->assertEquals('<a href="http://example.com" class="external" >Example</a>', $result);

        // r: 2
        $result = $link->renderLink('u|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('t:Example|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:2');
        $this->assertEquals('<a href="http://example.com" class="external" >Example</a>', $result);

        // r: 3
        $result = $link->renderLink('u|r:3');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|r:3');
//        $this->assertEquals('<span >http://example.com</span>', $result);
        $this->assertEquals('http://example.com', $result);

        $result = $link->renderLink('t:Example|r:3');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:3');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        // r: 4
        $result = $link->renderLink('u|r:4');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|r:4');
//        $this->assertEquals('<span >http://example.com</span>', $result);
        $this->assertEquals('http://example.com', $result);

        $result = $link->renderLink('t:Example|r:4');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:4');
//        $this->assertEquals('<span >http://example.com</span>', $result);
        $this->assertEquals('http://example.com', $result);

        // r: 5
        $result = $link->renderLink('u|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('t:Example|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:5');
        $this->assertEquals('', $result);
    }

    public function testRenderModePage() {
        $link = new Link(null, $this->sip, true);
        Store::setVar(TYPO3_PAGE_ID, 'firstPage', STORE_TYPO3);

        // r: default (0)
        $result = $link->renderLink('p');
        $this->assertEquals('<a href="?id=firstPage" class="internal" >?id=firstPage</a>', $result);

        $result = $link->renderLink('p:secondPage');
        $this->assertEquals('<a href="?id=secondPage" class="internal" >?id=secondPage</a>', $result);

        $result = $link->renderLink('t:Example');
        $this->assertEquals('', $result);

        $result = $link->renderLink('p:secondPage|t:Example');
        $this->assertEquals('<a href="?id=secondPage" class="internal" >Example</a>', $result);

        // r: 0
        $result = $link->renderLink('p|r:0');
        $this->assertEquals('<a href="?id=firstPage" class="internal" >?id=firstPage</a>', $result);

        $result = $link->renderLink('p:secondPage|r:0');
        $this->assertEquals('<a href="?id=secondPage" class="internal" >?id=secondPage</a>', $result);

        $result = $link->renderLink('t:Example|r:0');
        $this->assertEquals('', $result);

        $result = $link->renderLink('p:secondPage|t:Example|r:0');
        $this->assertEquals('<a href="?id=secondPage" class="internal" >Example</a>', $result);

        // r: 1
        $result = $link->renderLink('p|r:1');
        $this->assertEquals('<a href="?id=firstPage" class="internal" >?id=firstPage</a>', $result);

        $result = $link->renderLink('p:secondPage|r:1');
        $this->assertEquals('<a href="?id=secondPage" class="internal" >?id=secondPage</a>', $result);

        $result = $link->renderLink('t:Example|r:1');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('p:secondPage|t:Example|r:1');
        $this->assertEquals('<a href="?id=secondPage" class="internal" >Example</a>', $result);

        // r: 2
        $result = $link->renderLink('p|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('p:secondPage|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('t:Example|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('p:secondPage|t:Example|r:2');
        $this->assertEquals('<a href="?id=secondPage" class="internal" >Example</a>', $result);

        // r: 3
        $result = $link->renderLink('p|r:3');
//        $this->assertEquals('<span >?id=firstPage</span>', $result);
        $this->assertEquals('?id=firstPage', $result);

        $result = $link->renderLink('p:secondPage|r:3');
//        $this->assertEquals('<span >?id=secondPage</span>', $result);
        $this->assertEquals('?id=secondPage', $result);

        $result = $link->renderLink('t:Example|r:3');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('p:secondPage|t:Example|r:3');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        // r: 4
        $result = $link->renderLink('p|r:4');
//        $this->assertEquals('<span >?id=firstPage</span>', $result);
        $this->assertEquals('?id=firstPage', $result);

        $result = $link->renderLink('p:secondPage|r:4');
//        $this->assertEquals('<span >?id=secondPage</span>', $result);
        $this->assertEquals('?id=secondPage', $result);

        $result = $link->renderLink('t:Example|r:4');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('p:secondPage|t:Example|r:4');
//        $this->assertEquals('<span >?id=secondPage</span>', $result);
        $this->assertEquals('?id=secondPage', $result);

        // r: 5
        $result = $link->renderLink('p|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('p:secondPage|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('t:Example|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('p:secondPage|t:Example|r:5');
        $this->assertEquals('', $result);
    }

    public function testRenderModeMail() {
        $link = new Link(null, $this->sip, true);

        // r: default (0)
        $result = $link->renderLink('u');
        $this->assertEquals('', $result);

        $result = $link->renderLink('m:john@doe.com');
        $this->assertEquals('<a href="mailto:john@doe.com" >mailto:john@doe.com</a>', $result);

        $result = $link->renderLink('t:Example');
        $this->assertEquals('', $result);

        $result = $link->renderLink('m:john@doe.com|t:Example');
        $this->assertEquals('<a href="mailto:john@doe.com" >Example</a>', $result);

        // r: 0
        $result = $link->renderLink('u|r:0');
        $this->assertEquals('', $result);

        $result = $link->renderLink('m:john@doe.com|r:0');
        $this->assertEquals('<a href="mailto:john@doe.com" >mailto:john@doe.com</a>', $result);

        $result = $link->renderLink('t:Example|r:0');
        $this->assertEquals('', $result);

        $result = $link->renderLink('m:john@doe.com|t:Example|r:0');
        $this->assertEquals('<a href="mailto:john@doe.com" >Example</a>', $result);

        // r: 1
        $result = $link->renderLink('u|r:1');
        $this->assertEquals('', $result);

        $result = $link->renderLink('m:john@doe.com|r:1');
        $this->assertEquals('<a href="mailto:john@doe.com" >mailto:john@doe.com</a>', $result);

        $result = $link->renderLink('t:Example|r:1');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('m:john@doe.com|t:Example|r:1');
        $this->assertEquals('<a href="mailto:john@doe.com" >Example</a>', $result);

        // r: 2
        $result = $link->renderLink('u|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('m:john@doe.com|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('t:Example|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('m:john@doe.com|t:Example|r:2');
        $this->assertEquals('<a href="mailto:john@doe.com" >Example</a>', $result);

        // r: 3
        $result = $link->renderLink('u|r:3');
        $this->assertEquals('', $result);

        $result = $link->renderLink('m:john@doe.com|r:3');
//        $this->assertEquals('<span >mailto:john@doe.com</span>', $result);
        $this->assertEquals('mailto:john@doe.com', $result);

        $result = $link->renderLink('t:Example|r:3');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('m:john@doe.com|t:Example|r:3');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        // r: 4
        $result = $link->renderLink('u|r:4');
        $this->assertEquals('', $result);

        $result = $link->renderLink('m:john@doe.com|r:4');
//        $this->assertEquals('<span >mailto:john@doe.com</span>', $result);
        $this->assertEquals('mailto:john@doe.com', $result);

        $result = $link->renderLink('t:Example|r:4');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('m:john@doe.com|t:Example|r:4');
//        $this->assertEquals('<span >mailto:john@doe.com</span>', $result);
        $this->assertEquals('mailto:john@doe.com', $result);

        // r: 5
        $result = $link->renderLink('u|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('m:john@doe.com|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('t:Example|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('m:john@doe.com|t:Example|r:5');
        $this->assertEquals('', $result);
    }

    public function testRenderModeUrlPicture() {
        $link = new Link(null, $this->sip, true);

        // r: default (0)
        $result = $link->renderLink('u|P:picture.gif');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|P:picture.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > </a>', $result);

        $result = $link->renderLink('t:Example|P:picture.gif');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|P:picture.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > Example</a>', $result);

        // r: 0
        $result = $link->renderLink('u|r:0|P:picture.gif');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|r:0|P:picture.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > </a>', $result);

        $result = $link->renderLink('t:Example|r:0|P:picture.gif');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:0|P:picture.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > Example</a>', $result);

        // r: 1
        $result = $link->renderLink('u|r:1|P:picture.gif');
//        $this->assertEquals('<span ><img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > </span>', $result);
        $this->assertEquals('<img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > ', $result);

        $result = $link->renderLink('u:http://example.com|r:1|P:picture.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > </a>', $result);

        $result = $link->renderLink('t:Example|r:1|P:picture.gif');
//        $this->assertEquals('<span ><img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > Example</span>', $result);
        $this->assertEquals('<img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > Example', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:1|P:picture.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > Example</a>', $result);

        // r: 2
        $result = $link->renderLink('u|r:2|P:picture.gif');
        $this->assertEquals('', $result);

        //TODO: no link if text is empty - image is linked here: this is not what the user expects.
        $result = $link->renderLink('u:http://example.com|r:2|P:picture.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > </a>', $result);

        $result = $link->renderLink('t:Example|r:2|P:picture.gif');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:2|P:picture.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > Example</a>', $result);

        // r: 3:
        $result = $link->renderLink('u|r:3|P:picture.gif');
//        $this->assertEquals('<span ><img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > </span>', $result);
        $this->assertEquals('<img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > ', $result);

        $result = $link->renderLink('u:http://example.com|r:3|P:picture.gif');
//        $this->assertEquals('<span ><img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > </span>', $result);
        $this->assertEquals('<img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > ', $result);

        $result = $link->renderLink('t:Example|r:3|P:picture.gif');
//        $this->assertEquals('<span ><img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > Example</span>', $result);
        $this->assertEquals('<img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > Example', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:3|P:picture.gif');
//        $this->assertEquals('<span ><img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > Example</span>', $result);
        $this->assertEquals('<img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > Example', $result);

        // r: 4
        $result = $link->renderLink('u|r:4|P:picture.gif');
//        $this->assertEquals('<span ><img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > </span>', $result);
        $this->assertEquals('<img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > ', $result);

        $result = $link->renderLink('u:http://example.com|r:4|P:picture.gif');
//        $this->assertEquals('<span >http://example.com</span>', $result);
        $this->assertEquals('http://example.com', $result);

        $result = $link->renderLink('t:Example|r:4|P:picture.gif');
//        $this->assertEquals('<span ><img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > Example</span>', $result);
        $this->assertEquals('<img alt="Grafic: picture.gif" src="picture.gif" title="picture.gif" > Example', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:4|P:picture.gif');
//        $this->assertEquals('<span >http://example.com</span>', $result);
        $this->assertEquals('http://example.com', $result);

        // r: 5
        $result = $link->renderLink('u|r:5|P:picture.gif');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|r:5|P:picture.gif');
        $this->assertEquals('', $result);

        $result = $link->renderLink('t:Example|r:5|P:picture.gif');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:5|P:picture.gif');
        $this->assertEquals('', $result);
    }

    public function testLinkClass() {
    }

    protected function setUp() {
        parent::setUp();

        $this->store = Store::getInstance('', true);
        $this->sip = new Sip('fakesessionname', true);
        $this->sip->sipUniqId('badcaffee1234');

    }
}
